using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using HFShader.Rendering;
using HFShader.Camera;

namespace XnaTerrainShaderExample
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager mGraphicsManager;
        ContentManager mContentManager;

        Heightfield mHeightfield;

        GenericGameCamera mCamera;

        Vector3 mPlayerPosition;
        Vector3 mPlayerOrientation;

        public Game1()
        {
            mGraphicsManager = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // load content manager
            mContentManager = new ContentManager(this.Services);


            // init virtual player
            mPlayerPosition = new Vector3(-1000, 200, -1000);
            mPlayerOrientation = Vector3.Normalize(new Vector3(2, -1, 2));
            mPlayerOrientation.Normalize();


            // init camera
            mCamera = new GenericGameCamera();
            mCamera.mode = Generic_CamMode.firstPerson;
            mCamera.configuration.position = mPlayerPosition;
            mCamera.configuration.setDirection(mPlayerOrientation);
            mCamera.nearFarValues = new Vector2(1, 10000);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {

            // load heightfield description
            HeightfieldLoader loader = new HeightfieldLoader();
            HeightfieldParameter hfp = loader.loadHeightfieldSetup("DefaultMap1/map1", mContentManager, mGraphicsManager.GraphicsDevice);

            // create heightfield
            mHeightfield = new Heightfield(hfp, mContentManager, mGraphicsManager.GraphicsDevice, "Heightfield1");

        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            KeyboardState kbs = Keyboard.GetState();

            // Allows the game to exit
            if (kbs.IsKeyDown(Keys.Escape))
                this.Exit();



            // update virtual player
            float speedfactor = 1.0f;
          
            if (kbs.IsKeyDown(Keys.Left))
            {
                Matrix rot = Matrix.CreateRotationY(0.05f * speedfactor);
                mPlayerOrientation = Vector3.Transform(mPlayerOrientation, rot);
            }
            if (kbs.IsKeyDown(Keys.Right))
            {
                Matrix rot = Matrix.CreateRotationY(-0.05f * speedfactor);
                mPlayerOrientation = Vector3.Transform(mPlayerOrientation, rot);
            }
            if (kbs.IsKeyDown(Keys.Up))
            {
                mPlayerOrientation += new Vector3(0.0f, 0.04f * speedfactor, 0.0f);
                if (mPlayerOrientation.Y > 0.8f) mPlayerOrientation.Y = 0.8f;
                mPlayerOrientation.Normalize();
                System.Console.WriteLine(mPlayerOrientation.ToString());
            }
            if (kbs.IsKeyDown(Keys.Down))
            {
                mPlayerOrientation += new Vector3(0.0f, -0.04f * speedfactor, 0.0f);
                if (mPlayerOrientation.Y < -0.8f) mPlayerOrientation.Y = -0.8f;
                mPlayerOrientation.Normalize();
                System.Console.WriteLine(mPlayerOrientation.ToString());
            }

            if (kbs.IsKeyDown(Keys.W))
            {
                mPlayerPosition += Vector3.Multiply(mPlayerOrientation, 5.1f * speedfactor);
            }
            if (kbs.IsKeyDown(Keys.S))
            {
                mPlayerPosition -= Vector3.Multiply(mPlayerOrientation, 5.1f * speedfactor); 
            }



            // updtae camera
            mCamera.update(gameTime, mPlayerPosition, mPlayerOrientation);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // draw heightfield
            mHeightfield.draw(mCamera);

            base.Draw(gameTime);
        }
    }
}
